NEIGHBORS_4 = [
           (0, -1),         # noqa
    (-1, 0),       (1, 0),  # noqa
           (0,  1),         # noqa
]

NEIGHBORS_8 = [
    (-1, -1), (0, -1), (1, -1),  # noqa
    (-1,  0),          (1,  0),  # noqa
    (-1,  1), (0,  1), (1,  1),  # noqa
]

NEIGHBORS_N = {
    4: NEIGHBORS_4,
    8: NEIGHBORS_8,
}


def is_point_in_range(p, p_min=None, p_max=None) -> bool:
    """Check if point lays between two other points.

    Args:
        p: tuple (x, y)
        p_min (optional): min border point
        p_max (optional): max border point

    Returns:
        True if p_max >= p >= p_min
    """
    if p_min is None and p_max is None:
        return True

    x, y = p
    if p_min:
        x_min, y_min = p_min
        if x < x_min or y < y_min:
            return False
    if p_max:
        x_max, y_max = p_max
        if x > x_max or y > y_max:
            return False

    return True


def neighbors(p, n=4, p_min=None, p_max=None):
    """Point neighbor generator.

    Args:
        p: tuple (x, y)
        n: int 4 (no diagonal) or 8 (with diagonal)
        p_min (optional): min grid point, if not given infinite
        p_max (optional): max grid point, if not given infinite

    Yields:
        point (x, y)
    """
    neighbor_points = NEIGHBORS_N[n]
    x, y = p
    for dx, dy in neighbor_points:
        new_p = (x + dx, y + dy)
        if is_point_in_range(new_p, p_min, p_max):
            yield new_p



import pytest

from santa_helpers.neighbors import neighbors

EXAMPLES_PARSE_GRID_TO_DICT = (
    ((1, 1), 4, None, None, {(0, 1), (1, 0), (2, 1), (1, 2)}),
    ((0, 0), 4, (0, 0), None, {(0, 1), (1, 0)}),
    ((0, 0), 4, (0, 0), (0, 0), set()),
    ((0, 0), 8, None, None, {
        (-1, -1), (0, -1), (1, -1),
        (-1, 0), (1, 0),
        (-1,  1), (0,  1), (1,  1),
    }),
)


@pytest.mark.parametrize(
    'p,n,p_min,p_max,expected',
    EXAMPLES_PARSE_GRID_TO_DICT
)
def test_parse_grid_to_dict(p, n, p_min, p_max, expected):
    assert set(neighbors(p, n, p_min, p_max)) == expected



def parse_grid_to_dict(data: str) -> dict:
    """
    Parse grid given as a string to dictionary.
    k: coordinates (x, y)
    v: value

    Example:
        X.O    =>   { (0, 0): 'X', (1, 0): '.', (2, 0): 'O',
        ...           (0, 1): '.', (1, 1): '.', (2, 1): '.',
        ..O           (0, 2): '.', (1, 2): '.', (2, 2): 'O', }

    """
    return {
        (x, y): v
        for y, row in enumerate(data.strip().split('\n'))
        for x, v in enumerate(row.strip())
    }



import pytest

from santa_helpers.parse import parse_grid_to_dict

EXAMPLES_PARSE_GRID_TO_DICT = (
    ('.', {(0, 0): '.'}),
    ('A', {(0, 0): 'A'}),
    ('.-#', {(0, 0): '.', (1, 0): '-', (2, 0): '#'}),
    ("""
12
34
""", {(0, 0): '1', (1, 0): '2', (0, 1): '3', (1, 1): '4'}),
)


@pytest.mark.parametrize('data,expected', EXAMPLES_PARSE_GRID_TO_DICT)
def test_parse_grid_to_dict(data, expected):
    assert parse_grid_to_dict(data) == expected



"""Top-level package for santa-helpers."""

__author__ = """Magdalena Rother"""
__email__ = 'rother.magdalena@gmail.com'
__version__ = '0.0.1'

from .neighbors import neighbors  # noqa
from .parse import parse_grid_to_dict  # noqa
